import calculator_179d.SurrogateModelMetricConversion as smc
import calculator_179d.waterheater_attributes as wha
import numpy as np

def calculate_ext_wall_surface_area(calculator_user_inputs):

    if calculator_user_inputs["building_type"] == "small office":
        floor_to_floor_height= 10*0.3048 # floor height in m
    elif "retail" in calculator_user_inputs["building_type"]:
        floor_to_floor_height = 17*0.3048 # floor height for retail stripmall
    #floor area in m^2
    AF = (
        calculator_user_inputs['gross_floor_area']/
        calculator_user_inputs['number_of_floors']
    )
    #gross ext wall surface area in m^2
    ext_wall_surface_area_gross= (
        2*floor_to_floor_height*(
            np.sqrt(
                calculator_user_inputs['aspect_ratio']*AF
            )+
            np.sqrt(
                AF/calculator_user_inputs['aspect_ratio']
            )
        )
    )*calculator_user_inputs['number_of_floors']
    #opaque ext wall surface area in m^2
    ext_wall_surface_area = (
        (1-calculator_user_inputs['window_wall_ratio'])*
        ext_wall_surface_area_gross
    )

    return ext_wall_surface_area

def calculate_window_area(calculator_user_inputs):

    #opaque ext wall surface area
    ext_wall_surface_area = calculate_ext_wall_surface_area(calculator_user_inputs)
    #gross ext wall surface area 
    ext_wall_surface_area_gross = (
        ext_wall_surface_area/
        (1-calculator_user_inputs['window_wall_ratio'])
    )
    window_area = (
        calculator_user_inputs['window_wall_ratio']*
        ext_wall_surface_area_gross
    )

    return window_area


def calculate_roof_area(calculator_user_inputs):

    gross_floor_area = calculator_user_inputs['gross_floor_area']
    n_stories = calculator_user_inputs['number_of_floors']
    roof_area = (gross_floor_area/n_stories)

    return roof_area


def calculate_ACH_infiltration(calculator_user_inputs):

    bldg_type = calculator_user_inputs['building_type']
    if bldg_type == 'small office':
        AF = (
            calculator_user_inputs['gross_floor_area']/
            calculator_user_inputs['number_of_floors']
        )

        I = 0.0115824 
        ACH_infiltration = (
            (I*120*(
                np.sqrt(calculator_user_inputs['aspect_ratio']*AF)+
                np.sqrt(AF/calculator_user_inputs['aspect_ratio'])
                )
            )/(
                AF*calculator_user_inputs['number_of_floors']
            )
        )*calculator_user_inputs['number_of_floors']

    return ACH_infiltration


def calculate_ua_bldg(calculator_user_inputs):

    # calculate external wall surface area
    ext_wall_surface_area = calculate_ext_wall_surface_area(calculator_user_inputs)
    #calculate roof area
    roof_area = calculate_roof_area(calculator_user_inputs)
    # calculate window area
    window_area = calculate_window_area(calculator_user_inputs)

    #calculate ua_bldg
    ua_bldg = (
        roof_area*calculator_user_inputs['roof_thermal_perf_type'] +
        ext_wall_surface_area*calculator_user_inputs['wall_thermal_perf_type'] +
        window_area*calculator_user_inputs['window_u_factor']
    )

    return ua_bldg


def calculate_sa_to_vol_ratio(calculator_user_inputs):

    # calculate roof area
    roof_area = calculate_roof_area(calculator_user_inputs)
    #calculate surface to volume ratio
    sa_to_vol_ratio = (
        2. * ((
            calculator_user_inputs['aspect_ratio']/roof_area
            ) ** 0.5 + 
            (1./(calculator_user_inputs['aspect_ratio']*roof_area)
            ) **0.5) +
        (1./(10.*calculator_user_inputs["number_of_floors"]))
    )

    return sa_to_vol_ratio


def calculate_cooling_and_heating_cop(calculator_user_inputs):
    ##COP values from ahri
    ahri_args = {}
    #get cooling and heating cop
    cooling_ahri_metric = False
    heating_ahri_metric = False
    for feature,value in calculator_user_inputs.items():
        if feature in ['SEER','EER','SEER2','EER2']:
            ahri_args[feature] = value
            cooling_ahri_metric = True
        if feature in [
            'HSPF','HSPF2','HeatingCOP','boiler_average_efficiency','gas_coil_average_efficiency'
            ]:
            ahri_args[feature] = value
            heating_ahri_metric = True

        if cooling_ahri_metric and heating_ahri_metric:
            break
    ahri_args['hvac_system'] = calculator_user_inputs['hvac_system']
    [cooling_cop,heating_cop] = smc.AHRI2COP(ahri_args)

    return [cooling_cop,heating_cop]

def calculate_wh_ua_and_efficiency(calculator_user_inputs):
    ##COP values from ahri
    ahri_args = {}
    #get cooling and heating cop
    for feature,value in calculator_user_inputs.items():
        if feature in [
            'water_heating_standby_loss',
            'water_heating_thermal_efficiency',
            'water_heating_first_hour_rating',
            'water_heating_uef',
            'capacity'
        ]:
            ahri_args[feature] = value


    [ua_w_per_K,eta_burner_final] = wha.waterheater_attributes(ahri_args)

    return [ua_w_per_K,eta_burner_final]



import calculator_179d.proposed_electricity_179d as pe179d
import calculator_179d.baseline_electricity_179d as be179d
import calculator_179d.proposed_naturalgas_179d as pn179d
import calculator_179d.baseline_naturalgas_179d as bn179d
import json
import numpy as np
import os
import sys

# 4 prediction models used to compute proposed electricity and naturalgas
# and baseline electricity and naturalgas
# prediction model outputs are in GJ, function converts outputs to kBTU
def calculate_model_outputs(property_info):
    gJ_to_kBTU_conversion = 947.817

    # proposed and baseline electricity 179d

    # create proposed electricity model object based on bulding type, climate zone, and system type
    proposed_electricity_179d_obj = pe179d.proposed_electricity_179d(property_info)
    # estimate annual electricity using proposed model, and user calculator inputs
    proposed_electricity_179d_value = proposed_electricity_179d_obj.estimate_annual_electricity(
        property_info
    )*gJ_to_kBTU_conversion
    # create baseline model object based on bulding type, climate zone, and system type
    baseline_electricity_179d_obj = be179d.baseline_electricity_179d(property_info)
    # estimate baseline annual electricity withe baseline model and user calculator inputs
    baseline_electricity_179d_value = baseline_electricity_179d_obj.estimate_annual_electricity(
        property_info
    )*gJ_to_kBTU_conversion

    # create proposed model object based on bulding type, climate zone, and system type
    # no natural gas models for hvac systems with heat pump or have electric coil
    if (
        ('HP' not in property_info['hvac_system']) and 
        ('electric coil' not in property_info['hvac_system'])
    ):
        proposed_naturalgas_179d_obj = pn179d.proposed_naturalgas_179d(property_info)
        # estimate annual electricity using proposed model, and user calculator inputs
        proposed_naturalgas_179d_value = proposed_naturalgas_179d_obj.estimate_annual_naturalgas(
            property_info
        )*gJ_to_kBTU_conversion
        # create baseline model object based on bulding type, climate zone, and system type
        baseline_naturalgas_179d_obj = bn179d.baseline_naturalgas_179d(property_info)
        # estimate baseline annual electricity withe baseline model and user calculator inputs
        baseline_naturalgas_179d_value = baseline_naturalgas_179d_obj.estimate_annual_naturalgas(
            property_info
        )*gJ_to_kBTU_conversion

    else:
        proposed_naturalgas_179d_value = 0
        baseline_naturalgas_179d_value = 0

    model_outputs = [
        proposed_electricity_179d_value,
        baseline_electricity_179d_value,
        proposed_naturalgas_179d_value,
        baseline_naturalgas_179d_value
    ]
    #print("proposed_electricity gj:", model_outputs[0]/gJ_to_kBTU_conversion)
    return model_outputs

def calculate_energy_and_energy_savings(model_outputs,property_info):
    # proposed electricity consumption per sqft
    proposed_electricity_per_sqft = (
        model_outputs[0]/
        property_info['gross_floor_area']
    )
    # baseline electricity consumption per sqft
    baseline_electricity_per_sqft = (
        model_outputs[1]/
        property_info['gross_floor_area']
    )

    # proposed naturalgas per sqft
    proposed_naturalgas_per_sqft = (
        model_outputs[2]/
        property_info['gross_floor_area']
    )
    # baseline naturalgas per sqft
    baseline_naturalgas_per_sqft = (
        model_outputs[3]/
        property_info['gross_floor_area']
    )

    # Calculate total energy
    #electricity +  natural gas consumption in GJ
    proposed_total_energy = model_outputs[0]+model_outputs[2]
    proposed_total_energy_per_sqft = (
        proposed_total_energy/
        property_info['gross_floor_area']
    )
    baseline_total_energy = model_outputs[1]+model_outputs[3]
    baseline_total_energy_per_sqft = (
        baseline_total_energy/
        property_info['gross_floor_area']
    )

    # Calculate energy savings
    electricity_savings = model_outputs[2] - model_outputs[0]
    if electricity_savings<0:
        electricity_savings=0
    electricity_savings_per_sqft = (
        electricity_savings/
        property_info['gross_floor_area']
    )
    naturalgas_savings = model_outputs[3]-model_outputs[1]
    if naturalgas_savings<0:
        naturalgas_savings = 0
    naturalgas_savings_per_sqft = (
        naturalgas_savings/
        property_info['gross_floor_area']
    )
    total_energy_savings = baseline_total_energy - proposed_total_energy
    if total_energy_savings<0:
        total_energy_savings = 0
    total_energy_savings_per_sqft = (
        total_energy_savings/
        property_info['gross_floor_area']
    )

    return {
        'proposed_electricity_per_sqft':proposed_electricity_per_sqft,
        'baseline_electricity_per_sqft':baseline_electricity_per_sqft,
        'proposed_naturalgas_per_sqft':proposed_naturalgas_per_sqft,
        'baseline_naturalgas_per_sqft':baseline_naturalgas_per_sqft,
        'proposed_total_energy':proposed_total_energy,
        'proposed_total_energy_per_sqft':proposed_total_energy_per_sqft,
        'baseline_total_energy':baseline_total_energy,
        'baseline_total_energy_per_sqft':baseline_total_energy_per_sqft,
        'electricity_savings':electricity_savings,
        'electricity_savings_per_sqft':electricity_savings_per_sqft,
        'naturalgas_savings':naturalgas_savings,
        'naturalgas_savings_per_sqft':naturalgas_savings_per_sqft,
        'total_energy_savings':total_energy_savings,
        'total_energy_savings_per_sqft':total_energy_savings_per_sqft
    }

def calculate_cost_and_cost_savings(model_outputs,property_info):
    kBTU_to_kWh = 0.293014
    kBTU_to_thousand_cuft = 1./1039
    cents_to_dollar = 1./100

    # calculate costs
    # baseline costs
    baseline_179d_electricity_cost = (
        property_info['electricity_rate']*cents_to_dollar*
        model_outputs[1]*kBTU_to_kWh
    )
    baseline_179d_electricity_cost_per_sqft = (
        baseline_179d_electricity_cost/
        property_info['gross_floor_area']
    )
    baseline_179d_naturalgas_cost=(
        property_info['naturalgas_rate']*
        model_outputs[3]*kBTU_to_thousand_cuft
    )
    baseline_179d_naturalgas_cost_per_sqft = (
        baseline_179d_naturalgas_cost/
        property_info['gross_floor_area']
    )
    baseline_179d_totalcost = (
        baseline_179d_electricity_cost+
        baseline_179d_naturalgas_cost
    )
    baseline_179d_totalcost_per_sqft = (
        baseline_179d_totalcost/
        property_info['gross_floor_area']
    )

    # proposed costs
    proposed_179d_electricity_cost = (
        property_info['electricity_rate']*cents_to_dollar*
        model_outputs[0]*kBTU_to_kWh
    )
    proposed_179d_electricity_cost_per_sqft = (
        proposed_179d_electricity_cost/
        property_info['gross_floor_area']
    )
    proposed_179d_naturalgas_cost = (
        property_info['naturalgas_rate']*
        model_outputs[2]*kBTU_to_thousand_cuft
    )
    proposed_179d_naturalgas_cost_per_sqft = (
        proposed_179d_naturalgas_cost/
        property_info['gross_floor_area']
    )
    proposed_179d_totalcost = (
        proposed_179d_electricity_cost+
        proposed_179d_naturalgas_cost
    )
    proposed_179d_totalcost_per_sqft = (
        proposed_179d_totalcost/
        property_info['gross_floor_area']
    )

    # calculate cost savings
    electricity_cost_savings = (
        baseline_179d_electricity_cost-
        proposed_179d_electricity_cost
    )
    if electricity_cost_savings<0:
        electricity_cost_savings=0
    electricity_cost_savings_per_sqft = (
        electricity_cost_savings/
        property_info['gross_floor_area']
    )
    naturalgas_cost_savings = (
        baseline_179d_naturalgas_cost-
        proposed_179d_naturalgas_cost
    )
    if naturalgas_cost_savings<0:
        naturalgas_cost_savings=0
    naturalgas_cost_savings_per_sqft = (
        naturalgas_cost_savings/
        property_info['gross_floor_area']
    )
    total_cost_savings = (
        baseline_179d_totalcost-
        proposed_179d_totalcost
    )
    if total_cost_savings<0:
        total_cost_savings=0
    total_cost_savings_per_sqft = (
        total_cost_savings/
        property_info['gross_floor_area']
    )

    # Cost savings percentage
    Percent_Savings=round(
        (
            (baseline_179d_totalcost-proposed_179d_totalcost)/
            baseline_179d_totalcost
        )*100
    )
    if Percent_Savings<0:
        Percent_Savings=0
    Percent_Savings_Electricity=(
        round(
            ((baseline_179d_electricity_cost-
              proposed_179d_electricity_cost)/
            baseline_179d_electricity_cost)*100
        )
    )
    if Percent_Savings_Electricity<0:
        Percent_Savings_Electricity=0

    if (
        ('HP' not in property_info['hvac_system']) and
        ('electric coil' not in property_info['hvac_system'])
    ):
        Percent_Savings_Naturalgas=(
            round(
                ((baseline_179d_naturalgas_cost-
                proposed_179d_naturalgas_cost)/
                baseline_179d_naturalgas_cost)*100
            )
        )
    else:
        Percent_Savings_Naturalgas = 0
    if Percent_Savings_Naturalgas<0:
        Percent_Savings_Naturalgas=0
    Percent_Savings_per_sqft = (
        Percent_Savings/
        property_info['gross_floor_area']
    )

    return {
        'proposed_179d_electricity_cost':proposed_179d_electricity_cost,
        'baseline_179d_electricity_cost':baseline_179d_electricity_cost,
        'proposed_179d_electricity_cost_per_sqft':proposed_179d_electricity_cost_per_sqft,
        'baseline_179d_electricity_cost_per_sqft':baseline_179d_electricity_cost_per_sqft,
        'proposed_179d_naturalgas_cost':proposed_179d_naturalgas_cost,
        'proposed_179d_naturalgas_cost_per_sqft':proposed_179d_naturalgas_cost_per_sqft,
        'proposed_179d_totalcost':proposed_179d_totalcost,
        'proposed_179d_totalcost_per_sqft':proposed_179d_totalcost_per_sqft,
        'baseline_179d_totalcost':baseline_179d_totalcost,
        'baseline_179d_totalcost_per_sqft':baseline_179d_totalcost_per_sqft,
        'electricity_cost_savings':electricity_cost_savings,
        'electricity_cost_savings_per_sqft':electricity_cost_savings_per_sqft,
        'naturalgas_cost_savings':naturalgas_cost_savings,
        'naturalgas_cost_savings_per_sqft':naturalgas_cost_savings_per_sqft,
        'total_cost_savings':total_cost_savings,
        'total_cost_savings_per_sqft':total_cost_savings_per_sqft,
        'percent_savings_electricity':Percent_Savings_Electricity,
        'percent_savings_naturalgas':Percent_Savings_Naturalgas,
        'percent_savings':Percent_Savings,
        'percent_savings_total_per_sqft':Percent_Savings_per_sqft
    }


def calculate_savings(property_info):
    # calculate model outputs
    model_outputs = calculate_model_outputs(property_info)

    # output dictionary
    dict_outputs = {
        'proposed_179d_electricity':model_outputs[0],
        'baseline_179d_electricity':model_outputs[1],
        'proposed_179d_naturalgas':model_outputs[2],
        'baseline179d_natualgas':model_outputs[3]
    }

    # calculate energy and energy savings
    dict_outputs.update(calculate_energy_and_energy_savings(
        model_outputs,
        property_info
    )
    )

    # calculate cost and cost savings
    dict_outputs.update(
        calculate_cost_and_cost_savings(
            model_outputs,
            property_info
        )
    )

    BinSavings=np.arange(property_info['min_threshold_energy']*100,51, 1).tolist()
    # print(BinSavings)
    # print(dict_outputs['percent_savings'])

    RateList_energy=np.arange(
        property_info['energy_tax_deduction_rate_min'],
        property_info['energy_tax_deduction_rate_max']+property_info['increment_energy'],
        property_info['increment_energy']
    ).tolist()
    RateList_all=np.arange(
        property_info['all_179d_tax_deduction_rate_min'],
        property_info['all_179d_tax_deduction_rate_max'] + property_info['increment_all_179d'],
        property_info['increment_all_179d']
    ).tolist()

    if dict_outputs['percent_savings']>=property_info['min_threshold_energy']*100:
        Tax_deduction_Rate=RateList_energy[
            np.digitize(dict_outputs['percent_savings'],BinSavings)-1
        ]  # $/sqft
        dict_outputs['tax_deduction_rate_energy'] = Tax_deduction_Rate

    else:
        Tax_deduction_Rate=0
        dict_outputs['tax_deduction_rate_energy'] = Tax_deduction_Rate
        print("Minimum savings requirement not met and not qualified for 179D tax deduction")
    if dict_outputs['percent_savings']>=property_info['min_threshold_all_179d']*100:
        Tax_deduction_Rate_all=RateList_all[
            np.digitize(dict_outputs['percent_savings'],BinSavings)-1
        ]  # $/sqft

        dict_outputs['tax_deduction_rate_all'] = Tax_deduction_Rate_all
    else:
        dict_outputs['tax_deduction_rate_all'] = 0


    return dict_outputs





#if __name__ == "__main__":
#    json_file = sys.argv[1:][0]
#    #load json file given as argument in command line
#    with open(json_file) as f:
#        property_info = json.load(f)
#        #compute outputs and save in calculator_outputs.json
#        output = calculate_savings(property_info)
#        with open(r'./output_files/calculator_outputs.json', 'w') as fp:
#            json.dump(output, fp)



if __name__ == "__main__":
    json_file = sys.argv[1:][0]
    # load json file given as argument in command line
    f = open(json_file)
    calculator_user_inputs = json.load(f)
    # compute outputs and save in calculator_outputs.json
    calculate_savings(calculator_user_inputs)


import calculator_179d.calculate_common_features as ccf
import numpy as np
import joblib
import pkg_resources

class proposed_electricity_179d():
        def __init__(self, calculator_user_inputs):
            self.bldg_type = calculator_user_inputs['building_type']
            self.climate_zone = calculator_user_inputs['climate_zone']
            self.hvac_system = calculator_user_inputs['hvac_system']
            self.proposed_electricity_model = self.read_proposed_model()


        # pick the sklearn model based on bldg type, climate zone and hvac_system
        def read_proposed_model(self):
            model_file = (
                'proposed_electricity_' +
                self.bldg_type.replace(' ','_') + '_' +
                self.hvac_system.replace(' ', '_') +'_'
                'CZ'+self.climate_zone + '.pk'
            )

            model_path = pkg_resources.resource_filename(__name__, f"model_files/{model_file}")
            with open(model_path,'rb') as f:
                model = joblib.load(f)

            return model 


        # create input array used for model prediction
        def create_input_array(self, calculator_user_inputs):
            

            # calculated features - ua building, ACH infiltration, surface to volume ratio
            [cooling_cop,heating_cop] = ccf.calculate_cooling_and_heating_cop(calculator_user_inputs)
            ua_bldg = ccf.calculate_ua_bldg(calculator_user_inputs)
            ach_infiltration = ccf.calculate_ACH_infiltration(calculator_user_inputs)
            sa_to_vol_ratio = ccf.calculate_sa_to_vol_ratio(calculator_user_inputs)
            ext_wall_surface_area = ccf.calculate_ext_wall_surface_area(calculator_user_inputs)
            window_area = ccf.calculate_window_area(calculator_user_inputs)
            roof_area = ccf.calculate_roof_area(calculator_user_inputs)

            # create input array - same order as used during model development
            #include heating cop if hvac system is heat pump
            if "HP" in calculator_user_inputs["hvac_system"]:
                input_array = np.array(
                    [[
                        calculator_user_inputs['number_of_floors'],
                        calculator_user_inputs['gross_floor_area'],
                        ua_bldg,
                        calculator_user_inputs['window_wall_ratio'],
                        calculator_user_inputs['window_shgc'],
                        ach_infiltration,
                        calculator_user_inputs['proposed_lpd'],
                        cooling_cop,
                        heating_cop,
                        sa_to_vol_ratio,
                        calculator_user_inputs['aspect_ratio'],
                        ext_wall_surface_area,
                        roof_area,
                        window_area,
                        calculator_user_inputs['wall_thermal_perf_type'],
                        calculator_user_inputs['window_u_factor'],
                        calculator_user_inputs['roof_thermal_perf_type']
                    ]]
                )
            else:
                input_array = np.array(
                    [[
                        calculator_user_inputs['number_of_floors'],
                        calculator_user_inputs['gross_floor_area'],
                        ua_bldg,
                        calculator_user_inputs['window_wall_ratio'],
                        calculator_user_inputs['window_shgc'],
                        ach_infiltration,
                        calculator_user_inputs['proposed_lpd'],
                        cooling_cop,
                        sa_to_vol_ratio,
                        calculator_user_inputs['aspect_ratio'],
                        ext_wall_surface_area,
                        roof_area,
                        window_area,
                        calculator_user_inputs['wall_thermal_perf_type'],
                        calculator_user_inputs['window_u_factor'],
                        calculator_user_inputs['roof_thermal_perf_type']
                    ]]
                )
                 
            return input_array


        # Predict annual electricity 179d
        def estimate_annual_electricity(self, calculator_user_inputs):
            #create input array for prediction
            input_array = self.create_input_array(calculator_user_inputs)
            # predict floor area normalized annualy electricity 179d
            proposed_electricity_179d_value = self.proposed_electricity_model.predict(
                input_array
            )[0][0]

            return proposed_electricity_179d_value


# Commercial:

# Standby Loss (SL): The average hourly energy, expressed in Btu per hour, required to maintain the stored water temperature 
#                    based on a 70°F temperature differential between stored water and ambient room temperature.
#                    https://www.energystar.gov/products/water_heaters/commercial_water_heaters/key_product_criteria


# modeled UA units= W/K

 # Inputs:
    # water_heater_type
     # Commerical
         # standyloss
         # Thermal efficiency

         # Outputs:  ua_w_per_k, eta_burner_final

     # Residential
         # first_hour_rating [gallons]
         # uef
         # Capacity [btu/h]

         # Outputs: ua_w_per_k, eta_burner_final

import math

def waterheater_attributes(args):

    if (('water_heating_standby_loss' in args) and (args['water_heating_standby_loss']>0)):

        Btuperh2Watts = 0.29307107

        Temperaturedifferential = 70 #degF
        TemperaturedifferentialKelvin = Temperaturedifferential*(5/9)

        ua_w_per_k = (args['water_heating_standby_loss']*Btuperh2Watts)/TemperaturedifferentialKelvin

        eta_burner_final = args['water_heating_thermal_efficiency']/100

        return ua_w_per_k, eta_burner_final

    else:
        # define constant properties
        density = 8.2938 # lb/gal
        cp = 1.0007 # Btu/lb-F
        t_in = 58.0 # F
        t_env = 67.5 # F
        t = 125.0 # F

        ef = 0.9066 * args['water_heating_uef'] + 0.0711
        if ef >= 0.75:
          recovery_efficiency = 0.561 * ef + 0.439
        else:
          recovery_efficiency = 0.252 * ef + 0.608

        if (args['water_heating_first_hour_rating'] >= 0 and args['water_heating_first_hour_rating'] < 18):
            volume_drawn = 10.0 # gal
        elif (args['water_heating_first_hour_rating'] >= 18 and args['water_heating_first_hour_rating'] < 51):
            volume_drawn = 38.0 # gal
        elif (args['water_heating_first_hour_rating'] >= 51 and args['water_heating_first_hour_rating'] < 75):
            volume_drawn = 55.0 # gal
        elif (args['water_heating_first_hour_rating'] >= 75 and args['water_heating_first_hour_rating'] <= 130):
            volume_drawn = 84.0 # gal
        else:
            raise ValueError('first_hour_rating is beyond modeling range (< 130)')


        # calc ua_w_per_k and eta_burner_final
        draw_mass = volume_drawn * density # lb
        q_load = draw_mass * cp * (t - t_in) # Btu/day
        poww = args['water_heating_capacity']
        ua_btu_per_hr_r = ((recovery_efficiency / args['water_heating_uef']) - 1.0) / ((t - t_env) * (24.0 / q_load) - ((t - t_env) / (poww * args['water_heating_uef']))) # Btu/hr-F
        eta_burner_final = recovery_efficiency + ((ua_btu_per_hr_r * (t - t_env)) / poww) # conversion efficiency is slightly larger than recovery efficiency
        ua_w_per_k = ua_btu_per_hr_r/ 3.41 * 0.555556 # Btu/hr-R to W/K, 1 Btu/hr = 1/3.41 W, 1 R = 0.555556 K

        return ua_w_per_k, eta_burner_final



#----- Metric conversion from AHRI metrics to COP for use in the surrogate modeling and also include other metrics --------------

#------- Cooling
# Inputs: 
    # seer
    # eer
    # seer2
    # eer2
# Outputs:
#    COP

#------- Heating
# Inputs: 
    # HSPF
    # HSPF2
    # HeatingCOP (COP47)
    # Boiler Average Efficiency
    # Gas Coil Average Efficiency
# Outputs:
#    COP

# Usage:
    # args={'SEER':10,'HSPF':5}   
    # CoolingCOP,HeatingCOP=AHRI2COP(**args)


def AHRI2COP(args):

    if (('SEER' in args) and (args['SEER']>0)):

        coolingcop=0.2692*args['SEER']+0.2706

    elif (('SEER2' in args) and (args['SEER2']>0)):

        seer=args['SEER2']/0.95  # SEER2 to SEER conversion (SEER2/SEER=0.95)
        coolingcop = 0.2692*seer+0.2706

    elif (('EER' in args) and (args['EER']>0)):

        r = 0.12
        coolingcop = (args['EER'] / 3.413 + r) / (1 - r)

    elif (('EER2' in args) and (args['EER2']>0)):

        eer=args['EER2']/0.95 # EER2 to EER conversion (EER2/EER=0.95)
        r = 0.12
        coolingcop = (eer / 3.413 + r) / (1 - r)

    else:

        print('System type is different than DX')

    if (('electric coil') in args['hvac_system']):
        heatingcop = 0

    elif (('HSPF' in args) and (args['HSPF']>0)):
        heatingcop=0.0353*args['HSPF']*args['HSPF']+0.0331*args['HSPF']+0.9447

    elif (('HSPF2' in args) and (args['HSPF2']>0)):
        hspf = args['HSPF2']/0.84 # SEER2 to SEER conversion (HSPF2/HSPF=0.84)
        heatingcop = 0.0353*hspf**2+0.0331*hspf+0.9447

    elif (('heatingCOP' in args) and (args['heatingCOP']>0)):
        heatingcop = 1.48E-7 * args['heatingCOP'] * args['heating_capacity'] + 1.062 * args['heatingCOP']  # Heater capacity in Btu/hr  ToDo: check the heating_capacity as part of inputs with COP47

    elif (('boiler_average_efficiency' in args) and (args['boiler_average_efficiency']>0)):
        heatingcop = args['boiler_average_efficiency']

    elif (('gas_coil_average_efficiency' in args) and (args['gas_coil_average_efficiency']>0)):
        heatingcop = args['gas_coil_average_efficiency']

    else:
        print('Web tool efficiency metrics do not match the modeled')

    return coolingcop, heatingcop



import calculator_179d.calculate_common_features as ccf
import numpy as np
import joblib
import pkg_resources

class baseline_naturalgas_179d():
    def __init__(self, calculator_user_inputs):
        self.bldg_type = calculator_user_inputs['building_type']
        self.climate_zone = calculator_user_inputs['climate_zone']
        self.hvac_system = calculator_user_inputs['hvac_system']
        self.baseline_naturalgas_model = self.read_proposed_model()


    # pick the sklearn model based on bldg type, climate zone and hvac_system
    def read_proposed_model(self):
        model_file = (
                'baseline_naturalgas_' +
                self.bldg_type.replace(' ','_') + '_' +
                self.hvac_system.replace(' ', '_') +'_'
                'CZ'+self.climate_zone + '.pk'
            )

        model_path = pkg_resources.resource_filename(__name__, f"model_files/{model_file}")
        with open(model_path,'rb') as f:
            model = joblib.load(f)

        return model 


    # create input array used for model prediction
    def create_input_array(self, calculator_user_inputs):
        # calculated features - ua building, ACH infiltration, surface to volume ratio
        [cooling_cop,heating_cop] = ccf.calculate_cooling_and_heating_cop(calculator_user_inputs)
        [wh_ua_w_per_K, wh_eta_burner_final] = ccf.calculate_wh_ua_and_efficiency(
                 calculator_user_inputs
        )
        ua_bldg = ccf.calculate_ua_bldg(calculator_user_inputs)
        ach_infiltration = ccf.calculate_ACH_infiltration(calculator_user_inputs)
        sa_to_vol_ratio = ccf.calculate_sa_to_vol_ratio(calculator_user_inputs)
        sa_to_vol_ratio = ccf.calculate_sa_to_vol_ratio(calculator_user_inputs)
        ext_wall_surface_area = ccf.calculate_ext_wall_surface_area(calculator_user_inputs)
        window_area = ccf.calculate_window_area(calculator_user_inputs)
        roof_area = ccf.calculate_roof_area(calculator_user_inputs)

        # create input array - same order as used during model development
        input_array = np.array(
            [[
                calculator_user_inputs['number_of_floors'],
                calculator_user_inputs['gross_floor_area'],
                ua_bldg,
                calculator_user_inputs['window_wall_ratio'],
                calculator_user_inputs['window_shgc'],
                ach_infiltration,
                calculator_user_inputs['proposed_lpd'],
                heating_cop,
                sa_to_vol_ratio,
                calculator_user_inputs['aspect_ratio'],
                ext_wall_surface_area,
                roof_area,
                window_area,
                calculator_user_inputs['wall_thermal_perf_type'],
                calculator_user_inputs['window_u_factor'],
                calculator_user_inputs['roof_thermal_perf_type'],
                wh_eta_burner_final,
                wh_ua_w_per_K
            ]]
        )

        return input_array


    # Predict annual electricity 179d
    def estimate_annual_naturalgas(self, calculator_user_inputs):
        #create input array for prediction
        input_array = self.create_input_array(calculator_user_inputs)
        # predict floor area normalized annualy electricity 179d
        baseline_naturalgas_179d_value = self.baseline_naturalgas_model.predict(
            input_array
        )[0][0]


        return baseline_naturalgas_179d_value